<?php

namespace App\Controllers;

use App\Core\Database;
use App\Services\AuthService;
use App\Services\LogService;
use App\Services\LanguageService;
use App\Utils\DarkMode;

/**
 * Contrôleur pour la gestion des certificats.
 * (Création, révocation, affichage).
 */
class CertificateController
{
    private $db;
    private $authService;
    private $logService;
    private $langService;

    /**
     * Constructeur du CertificateController.
     */
    public function __construct()
    {
        $this->db = Database::getInstance();
        $this->authService = new AuthService($this->db);
        $this->logService = new LogService(APP_LOG_PATH);
        $this->langService = new LanguageService(APP_ROOT_DIR . '/src/Lang/');
    }

    /**
     * Affiche la liste des certificats, regroupés par périmètre fonctionnel.
     */
    public function index()
    {
        if (!$this->authService->isLoggedIn()) {
            header('Location: /login');
            exit();
        }

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();
        $userRole = $this->authService->getUserRole();

        // Récupérer les périmètres et les certificats
        // Joindre pour obtenir le nom du périmètre
        $certificates = $this->db->query("
            SELECT
                c.id, c.name, c.type, c.expiration_date, c.is_revoked, c.revoked_at,
                fp.name as perimeter_name
            FROM
                certificates c
            LEFT JOIN
                functional_perimeters fp ON c.functional_perimeter_id = fp.id
            ORDER BY
                fp.name IS NULL DESC, fp.name ASC, c.type DESC, c.expiration_date DESC
        ")->fetchAll();

        // Regrouper les certificats par périmètre fonctionnel
        $groupedCertificates = [];
        foreach ($certificates as $cert) {
            $perimeterName = $cert['perimeter_name'] ?? 'Certificats Root'; // Nom pour le groupe Root
            if (!isset($groupedCertificates[$perimeterName])) {
                $groupedCertificates[$perimeterName] = [];
            }
            $groupedCertificates[$perimeterName][] = $cert;
        }

        $successMessage = $_SESSION['success'] ?? null;
        unset($_SESSION['success']);
        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        require_once APP_ROOT_DIR . '/src/Views/certificates/index.php';
    }

    /**
     * Affiche le formulaire de création d'un nouveau certificat.
     */
    public function showCreateForm()
    {
        if (!$this->authService->isLoggedIn()) {
            header('Location: /login');
            exit();
        }

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();

        // Récupérer la liste des périmètres fonctionnels pour le sélecteur
        $perimeters = $this->db->query("SELECT id, name FROM functional_perimeters ORDER BY name ASC")->fetchAll();

        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        require_once APP_ROOT_DIR . '/src/Views/certificates/create.php';
    }

    /**
     * Traite la soumission du formulaire de création de certificat.
     */
    public function create()
    {
        if (!$this->authService->isLoggedIn() || $_SERVER['REQUEST_METHOD'] !== 'POST') {
            header('Location: /login');
            exit();
        }

        $subdomainName = trim($_POST['subdomain_name'] ?? '');
        $functionalPerimeterId = $_POST['functional_perimeter_id'] ?? null;
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $userId = $this->authService->getUserId();

        if (empty($subdomainName) || empty($functionalPerimeterId)) {
            $_SESSION['error'] = $this->langService->__('cert_create_error_empty_fields');
            header('Location: /certificates/create');
            exit();
        }

        // Récupérer le nom du périmètre fonctionnel pour le script shell
        $stmt = $this->db->prepare("SELECT name FROM functional_perimeters WHERE id = ?");
        $stmt->execute([$functionalPerimeterId]);
        $perimeter = $stmt->fetch();

        if (!$perimeter) {
            $_SESSION['error'] = $this->langService->__('cert_create_error_perimeter_not_found');
            header('Location: /certificates/create');
            exit();
        }
        $functionalPerimeterName = $perimeter['name'];

        // Extraire ROOT_DOMAIN du certificat CA racine pour SAN et OCSP
        $rootCaCertPath = ROOT_CA_PATH . '/certs/ca.cert.pem';
        if (!file_exists($rootCaCertPath)) {
            $this->logService->log('error', "Certificat CA racine non trouvé pour extraction ROOT_DOMAIN lors de la création de cert feuille.", $userId, $ipAddress);
            $_SESSION['error'] = $this->langService->__('cert_create_error_root_cert_missing'); // Needs this translation key
            header('Location: /certificates/create');
            exit();
        }
        $subjectCommand = "openssl x509 -noout -subject -in " . escapeshellarg($rootCaCertPath);
        $subjectLine = shell_exec($subjectCommand);
        $rootDomain = null;
        if ($subjectLine && preg_match('/CN\s*=\s*ca\.([^\/,\s]+)/', $subjectLine, $matches)) {
            $rootDomain = $matches[1];
        }
        if (empty($rootDomain)) {
            $this->logService->log('error', "Impossible d'extraire ROOT_DOMAIN du cert CA racine (pour SAN). Regex: '/CN\s*=\s*ca\.([^\/,\s]+)/' Sujet: " . ($subjectLine ?: 'vide'), $userId, $ipAddress);
            $_SESSION['error'] = $this->langService->__('cert_create_error_root_domain_extraction'); // Needs this translation key
            header('Location: /certificates/create');
            exit();
        }
        $this->logService->log('info', "ROOT_DOMAIN extrait pour SAN/OCSP: $rootDomain", $userId, $ipAddress);

        // Construire la valeur SAN
        $sanValue = "DNS:" . $subdomainName . "." . $functionalPerimeterName . "." . $rootDomain;

        // Récupérer OCSP_URL
        $ocspUrl = OCSP_URL; // Constante de app/src/config/app.php

        // Préparer la commande du script shell avec les variables d'environnement
        $scriptPath = SCRIPTS_PATH . '/create_cert.sh';
        $command = "OCSP_URL=" . escapeshellarg($ocspUrl) . " SAN=" . escapeshellarg($sanValue) . " " .
                   escapeshellcmd($scriptPath) . ' ' .
                   escapeshellarg($subdomainName) . ' ' .
                   escapeshellarg($functionalPerimeterName);

        $this->logService->log('info', "Tentative de création du certificat '$subdomainName' pour le périmètre '$functionalPerimeterName'. Commande: '$command'", $userId, $ipAddress);

        // Exécuter le script shell
        $output = shell_exec($command . ' 2>&1'); // Redirige stderr vers stdout

        // Vérifier le résultat du script
        // Le script create_cert.sh sort "Certificat '$CERT_BASE_NAME' créé avec succès : $CERT_FILE"
        // où $CERT_BASE_NAME est "${SUBDOMAIN_OR_CN_NAME}.${FUNCTIONAL_PERIMETER_NAME}"
        $certBaseNameForCheck = $subdomainName . '.' . $functionalPerimeterName;
        if (strpos($output, "Certificat '" . $certBaseNameForCheck . "' créé avec succès :") !== false) {
            // Extraire la date d'expiration du certificat créé (en lisant le fichier cert ou en estimant 1 an)
            $certFileName = "{$subdomainName}.{$functionalPerimeterName}.cert.pem";
            $fullCertPath = INTERMEDIATE_CA_PATH_BASE . "/{$functionalPerimeterName}/certs/{$certFileName}";

            $expirationDate = (new \DateTime('+1 year'))->format('Y-m-d H:i:s'); // Valeur par défaut
            if (file_exists($fullCertPath)) {
                $certInfo = shell_exec("openssl x509 -in " . escapeshellarg($fullCertPath) . " -noout -enddate 2>/dev/null | cut -d= -f2");
                $expirationTimestamp = strtotime($certInfo);
                if ($expirationTimestamp) {
                    $expirationDate = date('Y-m-d H:i:s', $expirationTimestamp);
                }
            }


            // Enregistrer le certificat dans la base de données
            $stmt = $this->db->prepare("INSERT INTO certificates (name, type, functional_perimeter_id, expiration_date) VALUES (?, ?, ?, ?)");
            $stmt->execute([$certFileName, 'simple', $functionalPerimeterId, $expirationDate]);

            $this->logService->log('info', "Certificat '{$certFileName}' créé et enregistré pour le périmètre '{$functionalPerimeterName}'.", $userId, $ipAddress);
            $_SESSION['success'] = $this->langService->__('cert_create_success');
        } else {
            $_SESSION['error'] = $this->langService->__('cert_create_error', ['output' => htmlspecialchars($output)]);
            $this->logService->log('error', "Échec création certificat '$subdomainName' pour périmètre '$functionalPerimeterName'. Output: $output", $userId, $ipAddress);
        }

        header('Location: /certificates');
        exit();
    }

    /**
     * Traite la révocation d'un certificat.
     */
    public function revoke()
    {
        if (!$this->authService->isLoggedIn() || $_SERVER['REQUEST_METHOD'] !== 'POST') {
            header('Location: /login');
            exit();
        }

        $certificateId = $_POST['certificate_id'] ?? null;
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $userId = $this->authService->getUserId();

        if (empty($certificateId)) {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_id_missing');
            header('Location: /certificates');
            exit();
        }

        // Récupérer les informations du certificat depuis la DB
        $stmt = $this->db->prepare("SELECT c.name, c.type, fp.name as perimeter_name FROM certificates c LEFT JOIN functional_perimeters fp ON c.functional_perimeter_id = fp.id WHERE c.id = ?");
        $stmt->execute([$certificateId]);
        $cert = $stmt->fetch();

        if (!$cert) {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_not_found');
            header('Location: /certificates');
            exit();
        }

        // Empêcher la révocation des certificats Root ou Intermédiaires via l'interface
        if ($cert['type'] === 'root' || $cert['type'] === 'intermediate') {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_ca_revocation');
            header('Location: /certificates');
            exit();
        }

        // Préparer le nom de base du certificat pour le script (sans l'extension .pem)
        $certBaseName = str_replace('.cert.pem', '.cert', $cert['name']);
        $functionalPerimeterName = $cert['perimeter_name'];

        // Vérifier si le certificat n'est pas déjà révoqué dans la DB
        if ($cert['is_revoked']) {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error_already_revoked');
            header('Location: /certificates');
            exit();
        }

        // Appeler le script shell de révocation
        $command = escapeshellcmd(SCRIPTS_PATH . '/revoke_cert.sh') . ' ' .
                   escapeshellarg($certBaseName) . ' ' .
                   escapeshellarg($functionalPerimeterName);

        $this->logService->log('info', "Tentative de révocation du certificat '{$cert['name']}' pour le périmètre '$functionalPerimeterName'. Commande: '$command'", $userId, $ipAddress);

        $output = shell_exec($command . ' 2>&1');

        if (strpos($output, "Certificat '$certBaseName' révoqué avec succès.") !== false) {
            // Mettre à jour le statut du certificat dans la base de données
            $stmt = $this->db->prepare("UPDATE certificates SET is_revoked = TRUE, revoked_at = NOW() WHERE id = ?");
            $stmt->execute([$certificateId]);

            $this->logService->log('info', "Certificat '{$cert['name']}' révoqué et enregistré en DB.", $userId, $ipAddress);
            $_SESSION['success'] = $this->langService->__('cert_revoke_success');
        } else {
            $_SESSION['error'] = $this->langService->__('cert_revoke_error', ['output' => htmlspecialchars($output)]);
            $this->logService->log('error', "Échec révocation certificat '{$cert['name']}': $output", $userId, $ipAddress);
        }

        header('Location: /certificates');
        exit();
    }

    /**
     * Gère le téléchargement des fichiers de certificats et clés.
     */
    public function download()
    {
        if (!$this->authService->isLoggedIn()) {
            header('Location: /login');
            exit();
        }

        // Récupérer les paramètres de la requête
        $type = $_GET['type'] ?? null;
        $fileName = $_GET['file'] ?? null;
        $perimeterName = $_GET['perimeter'] ?? null; // Utilisé pour intermédiaires et simples

        // Log de la tentative de téléchargement
        $userId = $this->authService->getUserId();
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $this->logService->log('info', "Download attempt: type='{$type}', file='{$fileName}', perimeter='{$perimeterName}'", $userId, $ipAddress);

        // Validation basique des paramètres
        if (empty($type) || empty($fileName)) {
            $_SESSION['error'] = 'Missing download parameters.';
            $this->logService->log('warn', "Missing download parameters.", $userId, $ipAddress);
            header('Location: /dashboard');
            exit();
        }

        // Sécurisation des noms de fichier et de périmètre
        if (basename($fileName) !== $fileName || ($perimeterName && basename($perimeterName) !== $perimeterName)) {
            $_SESSION['error'] = 'Invalid characters in file or perimeter name.';
            $this->logService->log('error', "Invalid characters in file or perimeter name for download.", $userId, $ipAddress);
            header('Location: /dashboard');
            exit();
        }

        $filePath = '';

        switch ($type) {
            case 'root':
                if ($fileName === 'ca.cert.pem') {
                    $filePath = ROOT_CA_PATH . '/certs/' . $fileName;
                } elseif ($fileName === 'ca.key.pem') {
                    if ($this->authService->getUserRole() === 'admin') {
                        $filePath = ROOT_CA_PATH . '/private/' . $fileName;
                    } else {
                         $_SESSION['error'] = 'Unauthorized to download root key.';
                         $this->logService->log('error', "Unauthorized attempt to download root CA key by user ID: {$userId}", $userId, $ipAddress);
                         header('Location: /dashboard');
                         exit();
                    }
                } else {
                    $_SESSION['error'] = 'Invalid file specified for root certificate type.';
                    $this->logService->log('warn', "Invalid file '{$fileName}' specified for root certificate type.", $userId, $ipAddress);
                    header('Location: /dashboard');
                    exit();
                }
                break;
            case 'intermediate':
                if (empty($perimeterName)) {
                    $_SESSION['error'] = 'Perimeter name missing for intermediate certificate.';
                    $this->logService->log('warn', "Perimeter name missing for intermediate certificate download.", $userId, $ipAddress);
                    header('Location: /dashboard');
                    exit();
                }
                $filePath = INTERMEDIATE_CA_PATH_BASE . '/' . $perimeterName . '/certs/' . $fileName;
                break;
            case 'simple':
                if (empty($perimeterName)) {
                    $_SESSION['error'] = 'Perimeter name missing for simple certificate.';
                    $this->logService->log('warn', "Perimeter name missing for simple certificate download.", $userId, $ipAddress);
                    header('Location: /dashboard');
                    exit();
                }
                $filePath = INTERMEDIATE_CA_PATH_BASE . '/' . $perimeterName . '/certs/' . $fileName;
                break;
            default:
                $_SESSION['error'] = 'Invalid certificate type for download.';
                $this->logService->log('warn', "Invalid certificate type for download: {$type}", $userId, $ipAddress);
                header('Location: /dashboard');
                exit();
        }

        if (!empty($filePath) && file_exists($filePath) && is_readable($filePath)) {
            $mimeType = 'application/octet-stream';
            $fileExtension = strtolower(pathinfo($filePath, PATHINFO_EXTENSION));

            if ($fileExtension === 'pem' || $fileExtension === 'key' || $fileExtension === 'crt' || $fileExtension === 'cer') {
                $mimeType = 'application/x-pem-file';
            }

            header('Content-Description: File Transfer');
            header('Content-Type: ' . $mimeType);
            header('Content-Disposition: attachment; filename="' . basename($filePath) . '"');
            header('Expires: 0');
            header('Cache-Control: must-revalidate');
            header('Pragma: public');
            header('Content-Length: ' . filesize($filePath));

            if (ob_get_level()) {
                ob_end_clean();
            }

            readfile($filePath);
            $this->logService->log('info', "File '{$filePath}' downloaded successfully.", $userId, $ipAddress);
            exit;
        } else {
            $_SESSION['error'] = 'File not found or not readable: ' . htmlspecialchars($fileName);
            $this->logService->log('error', "File not found or not readable for download: {$filePath} (Type: {$type}, File: {$fileName}, Perimeter: {$perimeterName})", $userId, $ipAddress);
            header('Location: /dashboard');
            exit();
        }
    }
}
