<?php

namespace App\Controllers;

use App\Core\Database;
use App\Services\AuthService;
use App\Services\LogService;
use App\Services\LanguageService;
use App\Utils\DarkMode;

/**
 * Contrôleur pour la gestion des périmètres fonctionnels.
 * (Création, affichage).
 */
class PerimeterController
{
    private $db;
    private $authService;
    private $logService;
    private $langService;

    /**
     * Constructeur du PerimeterController.
     */
    public function __construct()
    {
        $this->db = Database::getInstance();
        $this->authService = new AuthService($this->db);
        $this->logService = new LogService(APP_LOG_PATH);
        $this->langService = new LanguageService(APP_ROOT_DIR . '/src/Lang/');
    }

    /**
     * Affiche la liste des périmètres fonctionnels.
     */
    public function index()
    {
        if (!$this->authService->isLoggedIn()) {
            header('Location: /login');
            exit();
        }

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();

        $perimeters = $this->db->query("SELECT * FROM functional_perimeters ORDER BY name ASC")->fetchAll();

        $successMessage = $_SESSION['success'] ?? null;
        unset($_SESSION['success']);
        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        require_once APP_ROOT_DIR . '/src/Views/perimeters/index.php';
    }

    /**
     * Affiche le formulaire de création d'un nouveau périmètre fonctionnel.
     */
    public function showCreateForm()
    {
        if (!$this->authService->isLoggedIn() || $this->authService->getUserRole() !== 'admin') {
            $_SESSION['error'] = $this->langService->__('permission_denied');
            header('Location: /dashboard');
            exit();
        }

        global $translations;
        $currentLang = $this->langService->getLanguage();
        $darkModeClass = DarkMode::getBodyClass();

        $errorMessage = $_SESSION['error'] ?? null;
        unset($_SESSION['error']);

        require_once APP_ROOT_DIR . '/src/Views/perimeters/create.php';
    }

    /**
     * Traite la soumission du formulaire de création de périmètre fonctionnel.
     */
    public function create()
    {
        if (!$this->authService->isLoggedIn() || $this->authService->getUserRole() !== 'admin' || $_SERVER['REQUEST_METHOD'] !== 'POST') {
            $_SESSION['error'] = $this->langService->__('permission_denied');
            header('Location: /dashboard');
            exit();
        }

        $perimeterName = trim($_POST['name'] ?? '');
        $ipAddress = $_SERVER['REMOTE_ADDR'];
        $userId = $this->authService->getUserId();
        // La passphrase est optionnelle pour l'intermédiaire, mais le script attend l'argument.
        // Le script create_intermediate_cert.sh a été modifié pour accepter "EMPTY_STRING"
        // si aucune passphrase n'est fournie.
        $passphrase = trim($_POST['intermediate_passphrase'] ?? ''); // Assumons que cela vient du formulaire

        if (empty($perimeterName)) {
            $_SESSION['error'] = $this->langService->__('perimeter_create_error_empty_name');
            header('Location: /perimeters/create');
            exit();
        }

        // Vérifier si le périmètre existe déjà
        $stmt = $this->db->prepare("SELECT COUNT(*) FROM functional_perimeters WHERE name = ?");
        $stmt->execute([$perimeterName]);
        if ($stmt->fetchColumn() > 0) {
            $_SESSION['error'] = $this->langService->__('perimeter_create_error_exists');
            header('Location: /perimeters/create');
            exit();
        }

        // Extraire ROOT_DOMAIN du certificat CA racine
        $rootCaCertPath = ROOT_CA_PATH . '/certs/ca.cert.pem';
        $rootDomain = null;
        if (!file_exists($rootCaCertPath)) {
            $this->logService->log('error', "Certificat CA racine non trouvé à: $rootCaCertPath", $userId, $ipAddress);
            $_SESSION['error'] = $this->langService->__('perimeter_create_error_root_cert_missing'); // Nouvelle clé de traduction
            header('Location: /perimeters/create');
            exit();
        }

        $subjectCommand = "openssl x509 -noout -subject -in " . escapeshellarg($rootCaCertPath);
        $subjectLine = shell_exec($subjectCommand);

        if ($subjectLine && preg_match('/CN\s*=\s*ca\.([^\/,\s]+)/', $subjectLine, $matches)) {
            $rootDomain = $matches[1];
        }

        if (empty($rootDomain)) {
            $this->logService->log('error', "Impossible d'extraire ROOT_DOMAIN du certificat CA racine. Regex: '/CN\s*=\s*ca\.([^\/,\s]+)/' Sujet obtenu: " . ($subjectLine ?: 'vide ou non recupere'), $userId, $ipAddress);
            $_SESSION['error'] = $this->langService->__('perimeter_create_error_root_domain'); // Clé de traduction existante ou à ajouter
            header('Location: /perimeters/create');
            exit();
        }
        $this->logService->log('info', "ROOT_DOMAIN extrait avec succès: $rootDomain", $userId, $ipAddress);

        // Appeler le script shell pour créer le certificat intermédiaire
        // Retrieve OCSP_URL from defined constant
        $ocspUrl = OCSP_URL; // This constant is defined in app/src/config/app.php
        $sanValue = ''; // SAN is not typically needed for an intermediate CA, set to empty

        $passphraseArg = !empty($passphrase) ? $passphrase : "EMPTY_STRING";

        // Construct the shell command with environment variables
        // Note: escapeshellcmd is applied to the script path.
        // Environment variable values should also be safe.
        // Using escapeshellarg for values assigned to env vars is a good practice.
        $scriptPath = SCRIPTS_PATH . '/create_intermediate_cert.sh';
        $command = "OCSP_URL=" . escapeshellarg($ocspUrl) . " SAN=" . escapeshellarg($sanValue) . " " .
                   escapeshellcmd($scriptPath) . ' ' .
                   escapeshellarg($perimeterName) . ' ' .
                   escapeshellarg($passphraseArg) . ' ' .
                   escapeshellarg($rootDomain);

        $this->logService->log('info', "Tentative de création du périmètre '$perimeterName' et de son certificat intermédiaire. Commande: '$command'", $userId, $ipAddress);

        $output = shell_exec($command . ' 2>&1');

        // La condition de succès doit correspondre à la sortie du script.
        // Le script create_intermediate_cert.sh se termine par :
        // echo "Certificat Intermédiaire CA pour '$FUNCTIONAL_PERIMETER_NAME' créé : $INTERMEDIATE_CERT"
        // Nous allons donc chercher une partie de cette chaîne.
        if (strpos($output, "Certificat Intermédiaire CA pour '$perimeterName' créé") !== false) {
            // Enregistrer le périmètre dans la base de données
            $stmt = $this->db->prepare("INSERT INTO functional_perimeters (name, intermediate_cert_name) VALUES (?, ?)");
            $intermediateCertFileName = "intermediate.cert.pem"; // Nom générique du fichier pour l'intermédiaire
            $stmt->execute([$perimeterName, $intermediateCertFileName]);
            $perimeterId = $this->db->lastInsertId();

            // Enregistrer le certificat intermédiaire dans la table des certificats
            $fullCertPath = INTERMEDIATE_CA_PATH_BASE . "/{$perimeterName}/certs/intermediate.cert.pem";
            $expirationDate = (new \DateTime('+5 years'))->format('Y-m-d H:i:s'); // Valeur par défaut
            if (file_exists($fullCertPath)) {
                $certInfo = shell_exec("openssl x509 -in " . escapeshellarg($fullCertPath) . " -noout -enddate 2>/dev/null | cut -d= -f2");
                $expirationTimestamp = strtotime($certInfo);
                if ($expirationTimestamp) {
                    $expirationDate = date('Y-m-d H:i:s', $expirationTimestamp);
                }
            }

            $stmt = $this->db->prepare("INSERT INTO certificates (name, type, functional_perimeter_id, expiration_date) VALUES (?, ?, ?, ?)");
            $stmt->execute([$intermediateCertFileName, 'intermediate', $perimeterId, $expirationDate]);


            $this->logService->log('info', "Périmètre fonctionnel '$perimeterName' créé avec succès et certificat intermédiaire généré.", $userId, $ipAddress);
            $_SESSION['success'] = $this->langService->__('perimeter_create_success');
        } else {
            $_SESSION['error'] = $this->langService->__('perimeter_create_error', ['output' => htmlspecialchars($output)]);
            $this->logService->log('error', "Échec création périmètre '$perimeterName': $output", $userId, $ipAddress);
        }

        header('Location: /perimeters');
        exit();
    }
}
